{ config, pkgs, lib, ... }:

let
  lat = "51.081";
  lon = "13.728";
  altitude = "150";
  statsDir = "/tmp/dump1090";

  makeMlatClientService = args: {
    wantedBy = [ "multi-user.target" ];
    requires = [ "dump1090.service" ];
    serviceConfig = {
      User = "mlat-client";
      Group = "adsb";
      ProtectHome = true;
      Restart = "always";
      RestartSec = "10s";
    };
    path = with pkgs; [ mlat-client ];
    script = ''
      mlat-client --input-type beast --input-connect localhost:30005 --lat ${lat} --lon ${lon} --alt ${altitude} ${args}
    '';
  };
in {
  boot.blacklistedKernelModules = [
    # no watching TV intended
    "dvb_usb_rtl28xxu"
  ];
  sops.secrets = {
    "chaos-consulting/user".owner = "mlat-client";
  };
  users.users.dump1090 = {
    isSystemUser = true;
    group = "adsb";
  };
  users.groups.adsb = {};
  systemd.tmpfiles.rules = [
    "d ${statsDir} 0755 dump1090 adsb -"
  ];
  systemd.services.dump1090 = {
    wantedBy = [ "multi-user.target" ];
    serviceConfig = {
      ExecStart = "${pkgs.dump1090}/bin/dump1090 --quiet --modeac --forward-mlat --fix --stats-range --lon ${lon} --lat ${lat} --net --write-json ${statsDir}";
      User = "dump1090";
      Group = "adsb";
      ProtectSystem = "full";
      ProtectHome = true;
      Restart = "always";
      RestartSec = "10s";
    };
  };

  users.users.mlat-client = {
    isSystemUser = true;
    group = "adsb";
  };
  # Feeds adsbexchange.com, test at https://www.adsbexchange.com/myip/
  systemd.services.mlat-client-adsbexchange = makeMlatClientService "--server feed.adsbexchange.com:31090 --user C3D2";
  # Feeds https://adsb.chaos-consulting.de/map/
  systemd.services.mlat-client-chaos-consulting = makeMlatClientService "--server ${config.services.stunnel.clients.mlat-client-chaos-consulting.accept} --user \"$(cat ${config.sops.secrets."chaos-consulting/user".path})\"";

  # mlat-client-chaos-consulting needs ssl
  services.stunnel = {
    enable = true;
    clients.mlat-client-chaos-consulting = {
      accept = "127.0.0.1:3334";
      connect = "mlat.chaos-consulting.de:3334";
      verifyChain = false;
      verifyPeer = false;
    };
  };
  systemd.services.feed-chaos-consulting = {
    wantedBy = [ "multi-user.target" ];
    requires = [ "sbs2json.service" ];
    serviceConfig = {
      User = "mlat-client";
      Group = "adsb";
      ProtectHome = true;
      Restart = "always";
      RestartSec = "10s";
    };
    path = with pkgs; [ curl gzip ];
    script = ''
      while gzip -c < ${statsDir}/aircraft.json \
          | curl -s \
            -u "$(cat ${config.sops.secrets."chaos-consulting/user".path})" \
            -X POST \
            -H "Content-type: application/json" \
            -H "Content-encoding: gzip" \
            --data-binary @- \
            https://adsb.chaos-consulting.de/aircraftin/index.php
      do
        sleep 1
      done
    '';
  };

  users.users.dump1090-influxdb = {
    isSystemUser = true;
    group = "adsb";
  };
  systemd.services.dump1090-influxdb = {
    wantedBy = [ "multi-user.target" ];
    requires = [ "dump1090.service" ];
    serviceConfig = {
      ExecStart = "${pkgs.dump1090-influxdb}/bin/dump1090-influxdb";
      User = "dump1090-influxdb";
      Group = "adsb";
      ProtectSystem = "full";
      ProtectHome = true;
      Restart = "always";
      RestartSec = "10s";
    };
  };

  services.collectd.plugins.exec = ''
    Exec "${config.services.collectd.user}" "${pkgs.heliwatch.collectd-stats}/bin/collectd-stats"
  '';

  # serve dump1090 json data via http
  services.nginx = {
    enable = true;
    virtualHosts."radiobert.serv.zentralwerk.org" = {
      default = true;
      root = statsDir;
      extraConfig = ''
        autoindex on;
      '';
    };
  };
  systemd.services.nginx.serviceConfig.PrivateTmp = lib.mkForce false;

  environment.systemPackages = with pkgs; [
    dump1090
    #dump1090_rs
  ];
}
